/*
  This file is part of TALER
  (C) 2023, 2026 Taler Systems SA

  TALER is free software; you can redistribute it and/or modify it under the
  terms of the GNU Affero General Public License as published by the Free Software
  Foundation; either version 3, or (at your option) any later version.

  TALER is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
*/
/**
 * @file src/backend/taler-merchant-httpd_get-private-accounts-H_WIRE.c
 * @brief implement GET /accounts/$ID
 * @author Christian Grothoff
 */
#include "platform.h"
#include "taler-merchant-httpd_get-private-accounts-H_WIRE.h"
#include <taler/taler_json_lib.h>
#include <taler/taler_templating_lib.h>
#include "merchant-database/get_account.h"


/**
 * Handle a GET "/accounts/$ID" request.
 *
 * @param rh context of the handler
 * @param connection the MHD connection to handle
 * @param[in,out] hc context with further information about the request
 * @return MHD result code
 */
enum MHD_Result
TMH_private_get_accounts_ID (const struct TMH_RequestHandler *rh,
                             struct MHD_Connection *connection,
                             struct TMH_HandlerContext *hc)
{
  struct TMH_MerchantInstance *mi = hc->instance;
  const char *h_wire_s = hc->infix;
  struct TALER_MerchantWireHashP h_wire;
  struct TALER_MERCHANTDB_AccountDetails tp = { 0 };
  enum GNUNET_DB_QueryStatus qs;
  enum
  {
    POF_JSON,
    POF_TEXT
  } format;
  enum MHD_Result ret;

  GNUNET_assert (NULL != mi);
  GNUNET_assert (NULL != h_wire_s);
  if (GNUNET_OK !=
      GNUNET_STRINGS_string_to_data (h_wire_s,
                                     strlen (h_wire_s),
                                     &h_wire,
                                     sizeof (h_wire)))
  {
    GNUNET_break_op (0);
    return TALER_MHD_reply_with_error (connection,
                                       MHD_HTTP_BAD_REQUEST,
                                       TALER_EC_MERCHANT_GENERIC_H_WIRE_MALFORMED,
                                       h_wire_s);
  }
  qs = TALER_MERCHANTDB_get_account (TMH_db,
                                     mi->settings.id,
                                     &h_wire,
                                     &tp);
  if (0 > qs)
  {
    GNUNET_break (0);
    return TALER_MHD_reply_with_error (connection,
                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
                                       TALER_EC_GENERIC_DB_FETCH_FAILED,
                                       "get_account_serial");
  }
  if (0 == qs)
  {
    return TALER_MHD_reply_with_error (connection,
                                       MHD_HTTP_NOT_FOUND,
                                       TALER_EC_MERCHANT_GENERIC_ACCOUNT_UNKNOWN,
                                       hc->infix);
  }

  /* Determine desired output format from Accept header */
  {
    const char *mime;

    mime = MHD_lookup_connection_value (connection,
                                        MHD_HEADER_KIND,
                                        MHD_HTTP_HEADER_ACCEPT);
    if (NULL == mime)
      mime = "application/json";
    if (0 == strcmp (mime,
                     "*/*"))
      mime = "application/json";
    if (0 == strcmp (mime,
                     "application/json"))
    {
      format = POF_JSON;
    }
    else if (0 == strcmp (mime,
                          "text/plain"))
    {
      format = POF_TEXT;
    }
    else
    {
      GNUNET_break_op (0);
      ret = TALER_MHD_REPLY_JSON_PACK (
        connection,
        MHD_HTTP_NOT_ACCEPTABLE,
        GNUNET_JSON_pack_string ("hint",
                                 mime));
      goto cleanup;
    }
  }

  switch (format)
  {
  case POF_JSON:
    ret = TALER_MHD_REPLY_JSON_PACK (
      connection,
      MHD_HTTP_OK,
      GNUNET_JSON_pack_bool ("active",
                             tp.active),
      TALER_JSON_pack_full_payto ("payto_uri",
                                  tp.payto_uri),
      GNUNET_JSON_pack_data_auto ("h_wire",
                                  &tp.h_wire),
      GNUNET_JSON_pack_data_auto ("salt",
                                  &tp.salt),
      GNUNET_JSON_pack_allow_null (
        GNUNET_JSON_pack_string ("extra_wire_subject_metadata",
                                 tp.extra_wire_subject_metadata)),
      GNUNET_JSON_pack_allow_null (
        GNUNET_JSON_pack_string ("credit_facade_url",
                                 tp.credit_facade_url)));
    /* We do not return the credentials, as they may
       be sensitive */
    break;
  case POF_TEXT:
    {
      struct MHD_Response *resp;
      json_t *obj;
      unsigned int response_code = MHD_HTTP_OK;
      enum GNUNET_GenericReturnValue res;

      obj = GNUNET_JSON_PACK (
        GNUNET_JSON_pack_bool ("active",
                               tp.active),
        TALER_JSON_pack_full_payto ("payto_uri",
                                    tp.payto_uri));
      res = TALER_TEMPLATING_build (connection,
                                    &response_code,
                                    "account_status",
                                    mi->settings.id,
                                    NULL,
                                    obj,
                                    &resp);
      json_decref (obj);
      if (GNUNET_OK != res)
      {
        ret = TALER_MHD_REPLY_JSON_PACK (
          connection,
          MHD_HTTP_NOT_ACCEPTABLE,
          GNUNET_JSON_pack_string ("hint",
                                   "text/plain"));
        break;
      }
      TALER_MHD_add_global_headers (resp,
                                    false);
      GNUNET_break (MHD_YES ==
                    MHD_add_response_header (resp,
                                             MHD_HTTP_HEADER_CONTENT_TYPE,
                                             "text/plain"));
      ret = MHD_queue_response (connection,
                                response_code,
                                resp);
      MHD_destroy_response (resp);
    }
    break;
  }
cleanup:
  json_decref (tp.credit_facade_credentials);
  GNUNET_free (tp.extra_wire_subject_metadata);
  GNUNET_free (tp.payto_uri.full_payto);
  GNUNET_free (tp.credit_facade_url);
  return ret;
}


/* end of taler-merchant-httpd_get-private-accounts-H_WIRE.c */
